@@ -9,7 +9,14 @@ import pytest
 import re
 from flask import Flask
 import markupsafe
-from tests.test_utils import authenticate, check_xlation, json_authenticate, logout
+from tests.test_utils import (
+    authenticate,
+    check_xlation,
+    get_form_input,
+    init_app_with_options,
+    json_authenticate,
+    logout,
+)
 
 from flask_security import Security
 from flask_security.core import UserMixin
@@ -33,6 +40,8 @@ def test_registerable_flag(clients, app, get_message):
     response = clients.get("/register")
     assert b"<h1>Register</h1>" in response.data
     assert re.search(b'<input[^>]*type="email"[^>]*>', response.data)
+    assert get_form_input(response, "email") is not None
+    assert not get_form_input(response, "username")
 
     # Test registering is successful, sends email, and fires signal
     @user_registered.connect_via(app)
@@ -113,6 +122,39 @@ def test_registerable_flag(clients, app, get_message):
     assert b"Page 1" in response.data
 
 
+@pytest.mark.csrf(csrfprotect=True)
+def test_form_csrf(app, client):
+    # Test that CSRFprotect config catches CSRF requirement at unauth_csrf() decorator.
+    # Note that in this case - 400 is returned - though if CSRFprotect isn't enabled
+    # then CSRF errors are caught at form.submit time and a 200 with error values
+    # is returned.
+    response = client.post(
+        "/register",
+        data=dict(
+            email="csrf@example.com",
+            password="mypassword",
+            password_confirm="mypassword",
+        ),
+    )
+    assert response.status_code == 400
+    assert b"The CSRF token is missing" in response.data
+
+    response = client.get("/register")
+    csrf_token = get_form_input(response, "csrf_token")
+    response = client.post(
+        "/register",
+        data=dict(
+            email="csrf@example.com",
+            password="mypassword",
+            password_confirm="mypassword",
+            csrf_token=csrf_token,
+        ),
+        follow_redirects=False,
+    )
+    assert response.status_code == 302
+    assert response.location == "/"
+
+
 @pytest.mark.confirmable()
 @pytest.mark.app_settings(babel_default_locale="fr_FR")
 @pytest.mark.babel()
@@ -544,6 +586,14 @@ def test_username(app, client, get_message):
     )
 
 
+@pytest.mark.settings(username_enable=True)
+def test_username_template(app, client):
+    # verify template displays username option
+    response = client.get("/register")
+    username_field = get_form_input(response, "username")
+    assert username_field is not None
+
+
 @pytest.mark.settings(username_enable=True)
 @pytest.mark.unified_signin()
 def test_username_normalize(app, client, get_message):
@@ -882,3 +932,27 @@ def test_subclass(app, sqlalchemy_datastore):
     )
     assert response.status_code == 400
     assert "Really - don't start" in response.json["response"]["errors"][0]
+
+
+@pytest.mark.confirmable()
+@pytest.mark.settings(return_generic_responses=True)
+def test_my_mail_util(app, sqlalchemy_datastore):
+    # Test that with generic_responses - we still can get syntax/validation errors.
+    from flask_security import MailUtil, EmailValidateException
+
+    class MyMailUtil(MailUtil):
+        def validate(self, email):
+            if email.startswith("mike"):
+                raise EmailValidateException("No mikes allowed")
+
+    init_app_with_options(
+        app, sqlalchemy_datastore, **{"security_args": {"mail_util_cls": MyMailUtil}}
+    )
+
+    data = dict(
+        email="mike@lp.com",
+        password="awesome sunset",
+    )
+    client = app.test_client()
+    response = client.post("/register", data=data)
+    assert b"No mikes allowed" in response.data
